package ru.landgrafhomyak.serdha.api.v0.misc

import kotlin.jvm.JvmField
import kotlin.jvm.JvmName
import kotlinx.datetime.Instant
import kotlinx.datetime.LocalDate
import kotlinx.datetime.LocalDateTime
import kotlinx.datetime.LocalTime
import ru.landgrafhomyak.serdha.api.v0.table.RowId

/**
 * Descriptor of a database type.
 *
 * @param RuntimeType Type in programming language to which database type is converted.
 */
@Suppress("ClassName", "RemoveRedundantQualifierName")
public interface DatabaseType<@Suppress("unused") RuntimeType> {
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("name")
	public val name: String

	public abstract class _VirtualType<SRT, WRT : Any>(
		@Suppress("MemberVisibilityCanBePrivate") @JvmField
		public val wraps: DatabaseType<WRT>
	) : DatabaseType<SRT> {
		@Suppress("FunctionName")
		public abstract fun _unwrap(w: WRT): SRT

		@Suppress("FunctionName")
		public abstract fun _wrap(w: SRT): WRT

		@Suppress("FunctionName")
		public abstract fun _equals(l: SRT, r: SRT): Boolean

		@Suppress("FunctionName")
		public abstract fun _compare(l: SRT, r: SRT): Int
	}

	public interface _ROW_ID<T : RowId<TargetTableUserWrapper>?, TargetTableUserWrapper : Any> : DatabaseType<T>
	public interface ROW_ID<TargetTableUserWrapper : Any> : _ROW_ID<RowId<TargetTableUserWrapper>, TargetTableUserWrapper>
	public interface NULLABLE_ROW_ID<TargetTableUserWrapper : Any> : _ROW_ID<RowId<TargetTableUserWrapper>?, TargetTableUserWrapper>

	public interface BOOLEAN : DatabaseType<Boolean>
	public interface NULLABLE_BOOLEAN : DatabaseType<Boolean?>

	/**
	 * Descriptor of a raw binary data type for static type-checking.
	 * @see DatabaseTypesProvider.BINARY_DATA
	 */
	@OptIn(ExperimentalUnsignedTypes::class)
	public interface BINARY_DATA : DatabaseType<UByteArray> {
		/**
		 * Maximum count of bytes that can be stored by values of this type ot `null` if there is no limit.
		 */
		public val maxSizeOrNull: UInt?

		/**
		 * `true` if value can't hold less than [maxSizeOrNull][DatabaseType.BINARY_DATA.maxSizeOrNull] bytes.
		 */
		public val isFixedSize: Boolean
	}
}
