package ru.landgrafhomyak.serdha.api.v0.runtime

import ru.landgrafhomyak.serdha.api.v0.LowLevelApi
import ru.landgrafhomyak.serdha.api.v0.module.Module
import ru.landgrafhomyak.serdha.api.v0.table.RowId
import ru.landgrafhomyak.serdha.api.v0.table.Table
import ru.landgrafhomyak.serdha.api.v0.queries.CopyQuery
import ru.landgrafhomyak.serdha.api.v0.queries.DeleteQuery
import ru.landgrafhomyak.serdha.api.v0.queries.InsertQuery
import ru.landgrafhomyak.serdha.api.v0.queries.SelectQuery
import ru.landgrafhomyak.serdha.api.v0.queries.UpdateQuery

public interface SynchronizedDatabase<DatabaseUserExtension : Any> : Module<DatabaseUserExtension> {
	@Suppress("FunctionName")
	@LowLevelApi
	public suspend fun _startTransaction(): Transaction

	@Suppress("FunctionName")
	@LowLevelApi
	public suspend fun <QueryUserWrapper : Any> _autoTransactedSelect(query: SelectQuery<QueryUserWrapper>): _ParametersSetter<QueryUserWrapper, _ResultSet<QueryUserWrapper, Unit>>

	@Suppress("FunctionName")
	@LowLevelApi
	public suspend fun <QueryUserWrapper : Any> _autoTransactedInsert(query: InsertQuery<QueryUserWrapper>): _ParametersSetter<QueryUserWrapper, _ParametersSetter._Multi<InsertQuery.Creator.Scope.DataParam<QueryUserWrapper>, _ResultSet<QueryUserWrapper, _ResultSet<QueryUserWrapper, Unit>>?>>

	@Suppress("FunctionName")
	@LowLevelApi
	public suspend fun <QueryUserWrapper : Any> _autoTransactedCopy(query: CopyQuery<QueryUserWrapper>): _ParametersSetter<QueryUserWrapper, _ResultSet<QueryUserWrapper, _ResultSet<QueryUserWrapper, Unit>>?>

	@Suppress("FunctionName")
	@LowLevelApi
	public suspend fun <QueryUserWrapper : Any> _autoTransactedUpdate(query: UpdateQuery<QueryUserWrapper>): _ParametersSetter<QueryUserWrapper, _ResultSet<QueryUserWrapper, Unit>?>

	@Suppress("FunctionName")
	@LowLevelApi
	public suspend fun <QueryUserWrapper : Any> _autoTransactedDelete(query: DeleteQuery<QueryUserWrapper>): _ParametersSetter<QueryUserWrapper, _ResultSet<QueryUserWrapper, Unit>?>

	public interface AtomicScript<C, A, R> {
		public suspend fun executeTransaction(transaction: Transaction, context: C, args: A): R
	}

	public suspend fun <C, A, R> transaction(script: AtomicScript<C, A, R>, context: C, args: A): R

	@Suppress("FunctionName")
	@LowLevelApi
	public suspend fun _close()

	public fun <TableUserException : Any, V> mapOfRowIds(
		table: Table<TableUserException, *>,
		builder: (MutableMap<Row<TableUserException>, V>) -> Unit
	): Map<RowId<TableUserException>, V>

	public fun <TableUserException : Any, V> mutableMapOfRowIds(table: Table<TableUserException, *>): MutableMap<RowId<TableUserException>, V>
}