package ru.landgrafhomyak.serdha.api.v0.table

import ru.landgrafhomyak.serdha.api.v0.misc.BuiltinExpressionsProvider
import ru.landgrafhomyak.serdha.api.v0.misc.Column
import ru.landgrafhomyak.serdha.api.v0.misc.IntermediateColumn
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseType


public interface UpdateTableScope<TableNewUserExtension : Any, TableOldUserExtension : Any> : CreateTableScope<TableNewUserExtension> {
	public val oldExpressionBuilder: BuiltinExpressionsProvider<TableOldUserExtension>

	public fun <RT, DT : DatabaseType<RT>> keepColumn(c: Column<RT, DT, TableOldUserExtension>): Column<RT, DT, TableNewUserExtension>
	public fun <RT, DT : DatabaseType<RT>> keepColumnUntilUpgrading(c: Column<RT, DT, TableOldUserExtension>): Column<RT, DT, TableNewUserExtension>
	public fun <RT, DT : DatabaseType<RT>> renameAndKeepColumn(c: Column<RT, DT, TableOldUserExtension>, newName: String): Column<RT, DT, TableNewUserExtension>
	public fun deleteColumn(c: Column<*, *, TableOldUserExtension>)

	public fun keepIndex(i: Index<TableOldUserExtension>): Index<TableNewUserExtension>
	public fun renameAndKeepIndex(i: Index<TableOldUserExtension>, newName: String): Index<TableNewUserExtension>
	public fun deleteIndex(i: Index<TableOldUserExtension>)

	public fun keepCheck(i: CheckConstraint<TableOldUserExtension>): CheckConstraint<TableNewUserExtension>
	public fun renameAndKeepCheck(i: CheckConstraint<TableOldUserExtension>, newName: String): CheckConstraint<TableNewUserExtension>
	public fun deleteCheck(i: CheckConstraint<TableOldUserExtension>)

	public fun keepUniqueIndex(i: UniqueIndex<TableOldUserExtension>): UniqueIndex<TableNewUserExtension>
	public fun renameAndKeepUniqueIndex(i: UniqueIndex<TableOldUserExtension>, newName: String): UniqueIndex<TableNewUserExtension>
	public fun deleteUniqueIndex(i: UniqueIndex<TableOldUserExtension>)

	public fun <RT, DT : DatabaseType<RT>> keepDefaultValue(c: Column<RT, DT, TableNewUserExtension>, d: DefaultConstraint<RT, DT, TableOldUserExtension>): DefaultConstraint<RT, DT, TableNewUserExtension>
	public fun <RT, DT : DatabaseType<RT>> changeDefaultValue(c: Column<RT, DT, TableNewUserExtension>, d: DefaultConstraint<*, *, TableOldUserExtension>): DefaultConstraint<RT, DT, TableNewUserExtension>
	public fun deleteDefaultValue(i: DefaultConstraint<*, *, TableOldUserExtension>)

	public fun <TargetTableUserWrapper : Any> keepForeignRowReference(
		frr: ForeignRowReference<TableOldUserExtension, TargetTableUserWrapper>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<TableNewUserExtension, TargetTableUserWrapper>

	public fun <TargetTableOldUserWrapper : Any, TargetTableNewUserWrapper : Any> keepForeignRowReferenceToUpdatedTable(
		frr: ForeignRowReference<TableOldUserExtension, TargetTableOldUserWrapper>,
		updatedTable: Table<TargetTableNewUserWrapper, TargetTableOldUserWrapper>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<TableNewUserExtension, TargetTableNewUserWrapper>

	public fun <TargetTableUserWrapper : Any> renameAndKeepForeignRowReference(
		frr: ForeignRowReference<TableOldUserExtension, TargetTableUserWrapper>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<TableNewUserExtension, TargetTableUserWrapper>

	public fun <TargetTableOldUserWrapper : Any, TargetTableNewUserWrapper : Any> renameAndKeepForeignRowReferenceToUpdatedTable(
		frr: ForeignRowReference<TableOldUserExtension, TargetTableOldUserWrapper>,
		updatedTable: Table<TargetTableNewUserWrapper, TargetTableOldUserWrapper>,
		onDelete: ForeignRowReference.OnDeleteAction? = null
	): ForeignRowReference<TableNewUserExtension, TargetTableNewUserWrapper>

	public fun deleteForeignRowReference(frr: ForeignRowReference<TableOldUserExtension, *>)
}